@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Comments.pm 3742 2009-05-26 15:13:38Z jmarcotte $
+# $Id: Comments.pm 4075 2009-07-23 15:22:27Z plim $
 
 package MT::App::Comments;
 use strict;
@@ -44,6 +44,8 @@ sub init {
         recover          => \&recover,
         new_pw           => \&new_pw,
 
+        comment_listing    => \&comment_listing,
+
         # deprecated
         cmtr_name_js   => \&commenter_name_js,
         cmtr_status_js => \&commenter_status_js,
@@ -889,8 +891,6 @@ sub post {
         $app->translate( "An error occurred: [_1]", $app->errstr() ) )
         unless $comment;
 
-    $app->run_callbacks( 'api_post_save.comment', $app, $comment, $commenter );
-    
     my $remember = $q->param('bakecookie') || 0;
     $remember = 0 if $remember eq 'Forget Info';    # another value for '0'
     if ( $commenter && $remember ) {
@@ -974,6 +974,9 @@ sub post {
         }
         );
     if ( $comment->id && !$comment->is_junk ) {
+        
+        $app->run_callbacks( 'api_post_save.comment', $app, $comment, $commenter );
+        
         $app->log(
             {   message => $app->translate(
                     'Comment on "[_1]" by [_2].', $entry->title,
@@ -1473,6 +1476,61 @@ sub session_js {
     return undef;
 }
 
+sub comment_listing {
+    my ($app) = shift;
+    $app->{no_print_body} = 1;
+    $app->response_code(200);
+    $app->response_message('OK');
+    $app->send_http_header('text/javascript');
+
+    require MT::Entry;
+    require MT::Comment;
+    require MT::Template;
+    require MT::Template::Context;
+
+    my $entry_id = $app->param('entry_id');
+    return '1;' if ( !$entry_id );
+    my $entry = MT::Entry->load($entry_id);
+    return '1;' if ( !$entry );
+    my $offset = $app->param('offset');
+    $offset ||= 0;
+
+    if ( $offset !~ /^\d+$/ ) {
+        $offset = 0;
+    }
+    my $limit = $app->param('limit');
+    $limit ||= 100;
+    if ( $limit !~ /^\d+$/ ) {
+        $limit = 100;
+    }
+    my $direction = 'ascend';
+    if ( $app->param('direction') eq 'descend' ) {
+        $direction = 'descend';
+    }
+    my $method = $app->param('method');
+    $method ||= 'displayComments';
+    my $tmpl = MT::Template->load(
+        {   name    => 'Comment Listing',
+            blog_id => $entry->blog_id
+        }
+    );
+    return '1;' if ( !$tmpl );
+    my $total = MT::Comment->count( { entry_id => $entry_id, visible => 1 } );
+    my @comments = MT::Comment->load( { entry_id => $entry_id, visible => 1 },
+        { limit => $limit, offset => $offset, direction => $direction } );
+    my $ctx = MT::Template::Context->new;
+    $ctx->stash( 'entry',    $entry );
+    $ctx->stash( 'entry_id', $entry->id );
+    $ctx->stash( 'comments', \@comments );
+    $ctx->var( 'commentTotal',     $total );
+    $ctx->var( 'commentLimit',     $limit );
+    $ctx->var( 'commentOffset',    $offset );
+    $ctx->var( 'commentDirection', $direction );
+    $app->print( $tmpl->build($ctx) );
+    return 1;
+}
+
+
 # deprecated
 sub _commenter_status {
     my $app              = shift;
