@@ -1,13 +1,15 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Author.pm 3691 2009-05-12 18:06:46Z jmarcotte $
+# $Id: Author.pm 4046 2009-07-21 20:00:44Z plim $
 
 package MT::Author;
 
 use strict;
-use base qw( MT::Object MT::Scorable );
+
+use MT::Summary; # Holds MT::Summarizable
+use base qw( MT::Object MT::Scorable MT::Summarizable );
 
 __PACKAGE__->install_properties({
     column_defs => {
@@ -16,7 +18,7 @@ __PACKAGE__->install_properties({
         'nickname' => 'string(255)',
         'password' => 'string(60) not null',
         'type' => 'smallint not null',
-        'email' => 'string(75)',
+        'email' => 'string(127)',
         'url' => 'string(255)',
         'public_key' => 'text',
         'preferred_language' => 'string(50)',
@@ -64,6 +66,7 @@ __PACKAGE__->install_properties({
         basename => 1,
     },
     meta => 1,
+    summary => 1,
     child_classes => ['MT::Permission', 'MT::Association'],
     datasource => 'author',
     primary_key => 'id',
