@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Meta.pm 71460 2008-01-18 18:01:06Z ykerherve $
+# $Id: Meta.pm 5087 2009-11-20 19:56:19Z kshay $
 
 package MT::Meta;
 
@@ -92,12 +92,20 @@ our($Registry, $RegistryById);
 #--------------------------------------#
 # Public Class Methods
 
+sub _meta_args {
+    my $class = shift;
+    my ($pkg, $which) = @_;
+    $which ||= 'meta';
+    my $meth = $which . '_args';
+    $pkg->$meth;
+}
+
 sub install {
     my $class = shift;
-    my ($pkg, $params) = @_;
+    my ($pkg, $params, $which) = @_;
 
     ## add base class defs, if they exist
-    my $base_args = $pkg->meta_args;
+    my $base_args = $class->_meta_args($pkg, $which);
     if ($base_args) {
         while ( my ($k, $v) = each (%{ $base_args }) ) {
             $params->{$k} = $v;
@@ -122,7 +130,7 @@ sub install {
     }
 
     ## build subclass
-    $class->_build_subclass($pkg, $params);
+    $class->_build_subclass($pkg, $params, $which);
 
     return $params->{fields};
 }
@@ -183,7 +191,7 @@ sub has_own_metadata_of {
 sub normalize_type {
     my $pkg = shift;
     my ($type) = @_;
-    return $Types{ $TypesByName{ $type } } || TYPE_VBLOB;
+    return $Types{ $TypesByName{ $type } || TYPE_VBLOB };
 }
 
 #--------------------------------------#
@@ -207,9 +215,9 @@ sub _load_inheritance {
 
 sub _build_subclass {
     my $class = shift;
-    my ($pkg, $meta) = @_;
+    my ($pkg, $meta, $which) = @_;
 
-    my $subclass = $pkg->meta_pkg;
+    my $subclass = $pkg->meta_pkg($which);
     return unless $subclass;
 
     no strict 'refs'; ## no critic
@@ -289,11 +297,14 @@ you should not need to access I<MT::Meta> directly.
 These class methods allow you to retrieve information about the metadata
 defined for specific classes and metadata fields.
 
-=head2 MT::Meta->install($class, $params)
+=head2 MT::Meta->install($class, $params, [ $which ])
 
 Defines the set of metadata for the class I<$class> as described in the hash
 reference I<$params>, and configures I<$class> for use as a metadata host.
 
+If 'summary' is passed in the optional I<$which> argument, defines the set of 
+summary data, rather than metadata, for the class.
+
 Members of I<$params> can include:
 
 =over 4
