@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Context.pm 3525 2009-03-11 12:49:14Z fumiakiy $
+# $Id: Context.pm 4037 2009-07-21 13:41:49Z plim $
 
 package MT::Template::Context;
 
@@ -263,6 +263,9 @@ sub handler_for {
                 if (ref $code eq 'HASH') {
                     $code = $code->{code} ||= MT->handler_to_coderef($code->{handler});
                 }
+                elsif (defined $code and ! ref $code) {
+                    $code = MT->handler_to_coderef($code);
+                }
                 $str = $code->($str, $val, $ctx);
             }
         }
