@@ -1,9 +1,9 @@
 /*
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: mt.js 3528 2009-03-12 01:15:40Z fumiakiy $
+# $Id: mt.js 3988 2009-07-08 00:05:54Z mjacobs $
 */
 
 var pager;
@@ -2922,3 +2922,28 @@ function selectAll(id) {
     DOM.getElement( id ).focus();
     DOM.getElement( id ).select();
 }
+
+function removeAssetFromList(assetId) {
+    // remove the node list item
+    var Node = document.getElementById("list-asset-" + assetId);
+    if (Node) {
+        Node.parentNode.removeChild(Node);
+    } 
+
+    // remove the node's thumbnail
+    var Thumb = document.getElementById("list-image-" + assetId);
+    if (Thumb) {
+        Thumb.parentNode.removeChild(Thumb);
+    }
+
+    // get include_asset_ids and split it
+    var AssetList = document.getElementById("include_asset_ids").value;
+    var Assets = AssetList.split(",");
+    var NewAssetList = "";
+    for (var i = 0; i < Assets.length; i++) {
+        if (Assets[i] != assetId) {
+            NewAssetList = NewAssetList + Assets[i] + ",";
+        }
+    }
+    document.getElementById("include_asset_ids").value = NewAssetList;
+}
\ No newline at end of file
